//
//  CheckerViewController.m
//  ShopperChecker
//
//  Created by Felix Wang on 13/12/10.
//  Copyright 2010 Felix Wang. All rights reserved.
//

#import "CheckerViewController.h"
#import "ItemDetailViewController.h"


@implementation CheckerViewController
@synthesize tblView, clearButton;
@synthesize itemDetailViewController;
@synthesize appContext;
@synthesize checkerList;

// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
	self.navigationItem.title = @"Check List";
	ShopperCheckerAppDelegate *appDelegate = (ShopperCheckerAppDelegate *)[[UIApplication sharedApplication] delegate];
	appContext = [appDelegate managedObjectContext];
	ItemDetailViewController *_itemDetailViewController = [[ItemDetailViewController alloc] init];
	self.itemDetailViewController = _itemDetailViewController;
	[_itemDetailViewController release];
	
    [super viewDidLoad];
}

- (void)viewDidAppear:(BOOL)animated{
	if ([checkerList count] > 0) {
		[checkerList removeAllObjects];
	}
	NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
	NSEntityDescription *appEntity = [NSEntityDescription entityForName:@"ShoppingList"
												  inManagedObjectContext:appContext];
	[fetchRequest setEntity:appEntity];
	
	NSPredicate *checkerPredicate = [NSPredicate predicateWithFormat:@"status = %@", @"ON"];
	[fetchRequest setPredicate:checkerPredicate];
	
	NSError *error;
	NSMutableArray *_initCheckerList = [[NSMutableArray alloc] init];
	self.checkerList = _initCheckerList;
	[_initCheckerList release];
	
	NSArray *_shoppingList = [[NSArray alloc] init];
	_shoppingList = [appContext executeFetchRequest:fetchRequest error:&error];
	[self.checkerList addObjectsFromArray:_shoppingList];
	[fetchRequest release];
	
	//Sort the list
	NSSortDescriptor *sortDescriptorByName = [NSSortDescriptor sortDescriptorWithKey:@"name" ascending:YES];
	NSSortDescriptor *sortDescriptorByChecked = [NSSortDescriptor sortDescriptorWithKey:@"checked" ascending:YES];
	[checkerList sortUsingDescriptors:[NSArray arrayWithObject:sortDescriptorByName]];
	[checkerList sortUsingDescriptors:[NSArray arrayWithObject:sortDescriptorByChecked]];
	
	[tblView reloadData];
}

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (void)dealloc {
	[tblView release];
	[clearButton release];
	[checkerList release];
	[itemDetailViewController release];
    [super dealloc];
}

#pragma mark TableView Implementation
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
	static NSString *CellIdentifier = @"Cell";
	UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
	if (cell == nil)
	{
		cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier] autorelease];
	}
	
	UIImage *uncheckedBox = [UIImage imageNamed:@"Unchecked.png"];
	UIImage *checkedBox = [UIImage imageNamed:@"Checked.png"];
	ShoppingList *checkerItem = [checkerList objectAtIndex:indexPath.row];
	cell.textLabel.text = checkerItem.name;
	cell.detailTextLabel.text = checkerItem.desc;
	if ([checkerItem.checked isEqualToString:@"YES"]) {
		[cell.imageView setImage:checkedBox];
		cell.textLabel.textColor = [UIColor lightGrayColor];

	} else {
		[cell.imageView setImage:uncheckedBox];
		cell.textLabel.textColor = [UIColor blackColor];
	}
	cell.selectionStyle = UITableViewCellSelectionStyleNone;
	cell.accessoryType = UITableViewCellAccessoryDetailDisclosureButton;
	return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
	NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
	NSEntityDescription *entity = [NSEntityDescription entityForName:@"ShoppingList"
											  inManagedObjectContext:appContext];
	NSArray *selectedRow = [checkerList objectAtIndex:indexPath.row];
	NSPredicate *predicate = [NSPredicate predicateWithFormat:@"name=%@",[selectedRow valueForKey:@"name"]];
	[fetchRequest setPredicate:predicate];
	[fetchRequest setEntity:entity];
	NSError *error;
	NSArray *selectedItems = [appContext executeFetchRequest:fetchRequest error:&error];
	[fetchRequest release];
	if ([selectedItems count]>0) {
		ShoppingList *shoppingItem = [selectedItems objectAtIndex:0];
		if ([shoppingItem.checked isEqualToString:@"YES"]) {
			shoppingItem.checked = @"NO";
		} else {
			shoppingItem.checked = @"YES";
		}
		NSError *error;
		if (![appContext save:&error]) {
			NSLog(@"Whoops, couldn't save: %@", [error localizedDescription]);
		}	
	} else {
		NSLog(@"Can't get the shopping item from database.");
	}
	
	//Sort the list
	NSSortDescriptor *sortDescriptorByName = [NSSortDescriptor sortDescriptorWithKey:@"name" ascending:YES];
	NSSortDescriptor *sortDescriptorByChecked = [NSSortDescriptor sortDescriptorWithKey:@"checked" ascending:YES];
	[checkerList sortUsingDescriptors:[NSArray arrayWithObject:sortDescriptorByName]];
	[checkerList sortUsingDescriptors:[NSArray arrayWithObject:sortDescriptorByChecked]];
	
	[tblView reloadData];
}

- (void)tableView:(UITableView *)tableView accessoryButtonTappedForRowWithIndexPath:(NSIndexPath *)indexPath {
	ShoppingList *selectedItem = (ShoppingList *)[self.checkerList objectAtIndex:indexPath.row];
	[self.navigationController pushViewController:self.itemDetailViewController animated:YES];
	UIBarButtonItem *backButton = [[UIBarButtonItem alloc] initWithTitle:@"Back"
																   style:UIBarButtonItemStylePlain
																  target:nil 
																  action:nil];
	self.navigationItem.backBarButtonItem = backButton;
	[backButton release];
	UIBarButtonItem *editButton = [[UIBarButtonItem alloc] initWithTitle:@"Edit"
																   style:UIBarButtonItemStylePlain
																  target:self.itemDetailViewController
																  action:@selector(startEditing)];
	self.itemDetailViewController.navigationItem.rightBarButtonItem = editButton;
	self.itemDetailViewController.itemName.text = selectedItem.name;
	if ([selectedItem.checked isEqualToString:@"YES"]) {
		[self.itemDetailViewController.leftButton setTitle:@"Uncheck it" forState:UIControlStateNormal];
	} else {
		[self.itemDetailViewController.leftButton setTitle:@"Check it" forState:UIControlStateNormal];
	}
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	return [checkerList count];
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
	return 1;
}

#pragma mark Custom methods

- (void)clearTheCheckerList:(id)sender {
	for (ShoppingList *shoppingItem in checkerList) {
		if ([shoppingItem.checked isEqualToString:@"YES"]) {
			shoppingItem.status = @"OFF";
			NSError *error;
			if (![appContext save:&error]) {
				NSLog(@"Whoops, couldn't save: %@", [error localizedDescription]);
			}
		}
	}
	[self viewDidAppear:YES];
}

@end
