//
//  ItemDetailViewController.m
//  ShopperChecker
//
//  Created by Felix Wang on 25/12/10.
//  Copyright 2010 Felix Wang. All rights reserved.
//

#import "ItemDetailViewController.h"


@implementation ItemDetailViewController
@synthesize itemName, itemDesc;
@synthesize leftButton, rightButton;
@synthesize itemEditViewController;
@synthesize appContext;


/*
 // The designated initializer.  Override if you create the controller programmatically and want to perform customization that is not appropriate for viewDidLoad.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if ((self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil])) {
        // Custom initialization
    }
    return self;
}
*/


// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
	ItemEditViewController *_itemEditViewController = [[ItemEditViewController alloc] init];
	self.itemEditViewController = _itemEditViewController;
	[_itemEditViewController release];
	ShopperCheckerAppDelegate *appDelegate = (ShopperCheckerAppDelegate *)[[UIApplication sharedApplication] delegate];
	self.appContext = [appDelegate managedObjectContext];
    [super viewDidLoad];
	
	self.itemDesc.layer.cornerRadius = 5;
	self.itemDesc.layer.borderWidth = 1;
	self.itemDesc.layer.borderColor = [[UIColor lightGrayColor] CGColor];
	self.itemDesc.clipsToBounds = YES;
}

- (void)viewDidAppear:(BOOL)animated {
	NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
	NSEntityDescription *entity = [NSEntityDescription entityForName:@"ShoppingList"
											  inManagedObjectContext:appContext];
	NSPredicate *predicate = [NSPredicate predicateWithFormat:@"name=%@",self.itemName.text];
	[fetchRequest setPredicate:predicate];
	[fetchRequest setEntity:entity];
	NSError *error;
	NSArray *selectedItems = [appContext executeFetchRequest:fetchRequest error:&error];
	[fetchRequest release];
	if ([selectedItems count]>0) {
		ShoppingList *shoppingItem = [selectedItems objectAtIndex:0];
		self.itemDesc.text = shoppingItem.desc;	
	} else {
		NSLog(@"Can't get the shopping item from database.");
	}
	
}


/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	itemEditViewController = nil;
    
    // Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}


- (void)dealloc {
	[itemName release];
	[itemDesc release];
	[leftButton release];
	[rightButton release];
	[itemEditViewController release];
    [super dealloc];
}

#pragma mark Custom	methods

- (void)startEditing {
	[self.navigationController pushViewController:self.itemEditViewController animated:YES];
	self.itemEditViewController.itemName.text = self.itemName.text;
	self.itemEditViewController.itemDescription.text = self.itemDesc.text;
}

- (void)checkItemOff:(id)sender {
	NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
	NSEntityDescription *entity = [NSEntityDescription entityForName:@"ShoppingList"
											  inManagedObjectContext:appContext];
	NSPredicate *predicate = [NSPredicate predicateWithFormat:@"name=%@",self.itemName.text];
	[fetchRequest setPredicate:predicate];
	[fetchRequest setEntity:entity];
	NSError *error;
	NSArray *selectedItems = [appContext executeFetchRequest:fetchRequest error:&error];
	[fetchRequest release];
	if ([selectedItems count]>0) {
		ShoppingList *shoppingItem = [selectedItems objectAtIndex:0];
		if ([self.leftButton.titleLabel.text isEqualToString:@"Uncheck it"]) {
			shoppingItem.checked = @"NO";
		} else if ([self.leftButton.titleLabel.text isEqualToString:@"Check it"]) {
			shoppingItem.checked = @"YES";
		} else if ([self.leftButton.titleLabel.text isEqualToString:@"Take off list"]) {
			shoppingItem.status = @"OFF";
		} else if ([self.leftButton.titleLabel.text isEqualToString:@"Put on list"]) {
			shoppingItem.status = @"ON";
			shoppingItem.checked = @"NO";
		}
		NSError *error;
		if (![appContext save:&error]) {
			NSLog(@"Whoops, couldn't save: %@", [error localizedDescription]);
		}	
	} else {
		NSLog(@"Can't get the shopping item from database.");
	}
	[self.navigationController popViewControllerAnimated:YES];
}

- (void)deleteItem:(id)sender {
	NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
	NSEntityDescription *entity = [NSEntityDescription entityForName:@"ShoppingList"
											  inManagedObjectContext:appContext];
	NSPredicate *predicate = [NSPredicate predicateWithFormat:@"name=%@",self.itemName.text];
	[fetchRequest setPredicate:predicate];
	[fetchRequest setEntity:entity];
	NSError *error;
	NSArray *selectedItems = [appContext executeFetchRequest:fetchRequest error:&error];
	[fetchRequest release];
	if ([selectedItems count]>0) {
		[appContext deleteObject:[selectedItems objectAtIndex:0]];	
		NSError *error;
		if (![appContext save:&error]) {
			NSLog(@"Whoops, couldn't save: %@", [error localizedDescription]);
		}	
	} else {
		NSLog(@"Can't get the shopping item from database.");
	}
	[self.navigationController popViewControllerAnimated:YES];
}

@end
