    //
//  ListViewController.m
//  ShopperChecker
//
//  Created by Felix Wang on 13/12/10.
//  Copyright 2010 Felix Wang. All rights reserved.
//

#import "ListViewController.h"


@implementation ListViewController
@synthesize tblView, sBar;
@synthesize fetchedResultsController, managedObjectContext;
@synthesize searchedShoppingList;
@synthesize itemEditViewController, itemDetailViewController;


// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
	if (managedObjectContext==nil) {
		managedObjectContext = [(ShopperCheckerAppDelegate *)[[UIApplication sharedApplication] delegate] managedObjectContext];
	}
	self.navigationItem.title = @"Full List";
	ItemEditViewController *_itemEditViewController = [[ItemEditViewController alloc] init];
	self.itemEditViewController = _itemEditViewController;
	[_itemEditViewController release];
	ItemDetailViewController *_itemDetailViewController = [[ItemDetailViewController alloc] init];
	self.itemDetailViewController = _itemDetailViewController;
	[_itemDetailViewController release];
    [super viewDidLoad];

}

-(void)viewDidAppear:(BOOL)animated {
	[self.searchedShoppingList removeAllObjects];
	NSMutableArray *_searchedShoppingList = [[NSMutableArray alloc] init];
	self.searchedShoppingList = _searchedShoppingList;
	[_searchedShoppingList release];
	[self.searchedShoppingList addObjectsFromArray:[self getAllItems]];
	//Sort the mutable array ascending
	NSSortDescriptor *sortDescriptor = [NSSortDescriptor sortDescriptorWithKey:@"name" ascending:YES];
	[self.searchedShoppingList sortUsingDescriptors:[NSArray arrayWithObject:sortDescriptor]];
	
	[tblView reloadData];
	
	NSString *tmpSearchBartText = self.searchDisplayController.searchBar.text;
	if ([tmpSearchBartText length]>0) {
		[self searchBar:self.searchDisplayController.searchBar textDidChange:tmpSearchBartText];
		[self.searchDisplayController.searchResultsTableView reloadData];
	}
}

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
	self.itemEditViewController = nil;
	self.itemDetailViewController = nil;
}

- (void)viewDidUnload {
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}


- (void)dealloc {
	[sBar release];
	[tblView release];
	[fetchedResultsController release];
	[managedObjectContext release];
	[searchedShoppingList release];
	[itemEditViewController release];
	[itemDetailViewController release];
    [super dealloc];
}

#pragma mark TableView Implementation

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	return [self.searchedShoppingList count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
	static NSString *cellIdentifier = @"Cell";
	UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellIdentifier];
	if (cell == nil) {
		cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:cellIdentifier] autorelease];
	}
	UIImage *addToList = [UIImage imageNamed:@"AddToList.png"];
	UIImage *removeFromList = [UIImage imageNamed:@"RemoveFromList.png"];
	if ([self.searchedShoppingList count]>0) {
		ShoppingList *item = [self.searchedShoppingList objectAtIndex:indexPath.row];
		cell.textLabel.text = item.name;
		cell.detailTextLabel.text = item.desc;
		if ([item.status isEqualToString:@"ON"]) {
			[cell.imageView setImage:removeFromList];
		}else {
			[cell.imageView setImage:addToList];
		}
	}
	cell.selectionStyle = UITableViewCellSelectionStyleNone;
	cell.accessoryType = UITableViewCellAccessoryDetailDisclosureButton;
	return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
	
	NSArray *selectedItems = [self getItemByName:[[searchedShoppingList objectAtIndex:indexPath.row] valueForKey:@"name"]];
	
	if ([selectedItems count]>0) {
		ShoppingList *shoppingItem = [selectedItems objectAtIndex:0];
		if ([shoppingItem.status isEqualToString:@"ON"]) {
			shoppingItem.status = @"OFF";
		} else {
			shoppingItem.status = @"ON";
			shoppingItem.checked = @"NO";
		}
		NSError *error;
		if (![managedObjectContext save:&error]) {
			NSLog(@"Whoops, couldn't save: %@", [error localizedDescription]);
		}	
	} else {
		NSLog(@"Can't get the shopping item from database.");
	}
	
	NSSortDescriptor *sortDescriptorByName = [NSSortDescriptor sortDescriptorWithKey:@"name" ascending:YES];
	[searchedShoppingList sortUsingDescriptors:[NSArray arrayWithObject:sortDescriptorByName]];
	
	[tblView reloadData];
	
	NSString *tmpSearchBartText = self.searchDisplayController.searchBar.text;
	if ([tmpSearchBartText length]>0) {
		[self searchBar:self.searchDisplayController.searchBar textDidChange:tmpSearchBartText];
		[self.searchDisplayController.searchResultsTableView reloadData];
	}
}

- (void)tableView:(UITableView *)tableView accessoryButtonTappedForRowWithIndexPath:(NSIndexPath *)indexPath {
	ShoppingList *selectedItem = (ShoppingList *)[self.searchedShoppingList objectAtIndex:indexPath.row];
	[self.navigationController pushViewController:self.itemDetailViewController animated:YES];
	UIBarButtonItem *backButton = [[UIBarButtonItem alloc] initWithTitle:@"Back"
																   style:UIBarButtonItemStylePlain
																  target:nil 
																  action:nil];
	self.navigationItem.backBarButtonItem = backButton;
	[backButton release];
	UIBarButtonItem *editButton = [[UIBarButtonItem alloc] initWithTitle:@"Edit"
																   style:UIBarButtonItemStylePlain
																  target:self.itemDetailViewController
																  action:@selector(startEditing)];
	self.itemDetailViewController.navigationItem.rightBarButtonItem = editButton;
	self.itemDetailViewController.itemName.text = selectedItem.name;
	if ([selectedItem.status isEqualToString:@"OFF"]) {
		[self.itemDetailViewController.leftButton setTitle:@"Put on list" forState:UIControlStateNormal];
	} else {
		[self.itemDetailViewController.leftButton setTitle:@"Take off list" forState:UIControlStateNormal];
	}
	
}

#pragma mark UISearchBarDelegate
- (void)searchBarTextDidBeginEditing:(UISearchBar *)searchBar
{
	// only show the status bar’s cancel button while in edit mode
	sBar.showsCancelButton = YES;
	sBar.autocorrectionType = UITextAutocorrectionTypeNo;
	// flush the previous search content
	[self.searchedShoppingList removeAllObjects];
	[tblView reloadData];
}

- (void)searchBarTextDidEndEditing:(UISearchBar *)searchBar
{
	sBar.showsCancelButton = NO;
}

- (void)searchBarCancelButtonClicked:(UISearchBar *)searchBar {
	// Here to repopulate the list.
	[self.searchedShoppingList addObjectsFromArray:[self getAllItems]];
	//Sort the mutable array ascending
	NSSortDescriptor *sortDescriptor = [NSSortDescriptor sortDescriptorWithKey:@"name" ascending:YES];
	[self.searchedShoppingList sortUsingDescriptors:[NSArray arrayWithObject:sortDescriptor]];
	[tblView reloadData];	
}

- (void)searchBar:(UISearchBar *)searchBar textDidChange:(NSString *)searchText
{
	[self.searchedShoppingList removeAllObjects];// remove all data that belongs to previous search
	if([searchText isEqualToString:@""] || searchText==nil){
		[self.searchedShoppingList addObjectsFromArray:[self getAllItems]];
		//Sort the mutable array ascending
		NSSortDescriptor *sortDescriptor = [NSSortDescriptor sortDescriptorWithKey:@"name" ascending:YES];
		[self.searchedShoppingList sortUsingDescriptors:[NSArray arrayWithObject:sortDescriptor]];
		[tblView reloadData];	
	} else {
		for(ShoppingList *item in [self getAllItems])
		{
			NSAutoreleasePool *pool = [[NSAutoreleasePool alloc]init];
			NSRange r = [[item.name lowercaseString] rangeOfString:[searchText lowercaseString]];
			if(r.location != NSNotFound)
			{
				if(r.location== 0)//that is we are checking only the start of the names.
				{
					[self.searchedShoppingList addObject:item];
				}
			}
			[pool release];
		}
		//Sort the mutable array ascending
		NSSortDescriptor *sortDescriptor = [NSSortDescriptor sortDescriptorWithKey:@"name" ascending:YES];
		[self.searchedShoppingList sortUsingDescriptors:[NSArray arrayWithObject:sortDescriptor]];
		[tblView reloadData];
	}
}


#pragma mark Custom methods

-(void)newShoppingItem:(id)sender {
	[self.navigationController pushViewController:self.itemEditViewController animated:YES];
	UIBarButtonItem *cancelButton = [[UIBarButtonItem alloc] initWithTitle:@"Cancel" style:UIBarButtonItemStylePlain target:nil action:nil];
	self.navigationItem.backBarButtonItem = cancelButton;
	[cancelButton release];
	self.itemEditViewController.itemName.text = nil;
	self.itemEditViewController.itemDescription.text = nil;
}

-(NSArray *)getAllItems {
	NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
	NSEntityDescription *entity = [NSEntityDescription entityForName:@"ShoppingList"
											  inManagedObjectContext:managedObjectContext];
	[fetchRequest setEntity:entity];
	NSError *error;
	NSArray *selectedItems = [managedObjectContext executeFetchRequest:fetchRequest error:&error];
	[fetchRequest release];
	return selectedItems;
}

-(NSArray *)getItemByName:(NSString *)itemName {
	NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
	NSEntityDescription *entity = [NSEntityDescription entityForName:@"ShoppingList"
											  inManagedObjectContext:managedObjectContext];
	NSPredicate *predicate = [NSPredicate predicateWithFormat:@"name=%@", itemName];
	[fetchRequest setPredicate:predicate];
	[fetchRequest setEntity:entity];
	NSError *error;
	NSArray *selectedItems = [managedObjectContext executeFetchRequest:fetchRequest error:&error];
	[fetchRequest release];
	return selectedItems;
}

@end
